// Initialisation de l'EEPROM pour le sidaster
//--------------------------------------------


#include <Wire.h> //I2C library
#include <LiquidCrystal_I2C.h>
#if defined(ARDUINO) && ARDUINO >= 100
#define printByte(args)  write(args);
#else
#define printByte(args)  print(args,BYTE);
#endif

LiquidCrystal_I2C lcd(0x27,20,4);       // sLCD adress 0x27, 16 car, 2 lignes



#define MAXPROG 128
#define MAXBANK 4
#define PRESETSIZE  64
#define BANKSIZE  8192

char nompreset[17];   
static byte coarse1 = 16;
static byte coarse2 = 16;
static byte coarse3 = 16;
static byte fine1 = 0;
static byte fine2 = 0;
static byte fine3 = 0;
static word	pw1 = 0x800;
static word	pw2 = 0x800;
static word	pw3 = 0x800;
static byte	ctrl1 = 0X40;
static byte	ctrl2 = 0X00;
static byte	ctrl3 = 0X00;
static byte	ad1 = 0x00;
static byte	ad2 = 0x00;
static byte	ad3 = 0x00;
static byte	sr1 = 0xF0;
static byte	sr2 = 0xF0;
static byte	sr3 = 0xF0;
static word	freq = 0x7FF;
static byte	res = 0x00;
static byte	mode = 0x0F;
static byte Lfo1Speed = 0x10;
static byte Lfo2Speed = 0x10;
static byte Lfo1Wave = 0x01;
static byte Lfo2Wave = 0x01;
static byte Lfo1Dest = 0x00;
static byte Lfo2Dest = 0x00;
static byte Lfo1Level = 0x00;
static byte Lfo2Level = 0x00;



void eewrite_byte( int deviceaddress, unsigned int eeaddress, byte data ) 
{
int rdata = data;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.write(rdata);
  Wire.endTransmission();
  delay(5);
}

byte eeread_byte( int deviceaddress, unsigned int eeaddress ) {
  byte rdata = 0xFF;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.endTransmission();
  Wire.requestFrom(deviceaddress,1);
  if (Wire.available()) rdata = Wire.read();
    return rdata;
}

void setup() 
{
  //Wire.begin(); // initialise the connection
  lcd.init();             // init du lcd 
  lcd.backlight();        // Retroeclairage
  lcd.clear();            // Effacement de lecran
  lcd.setCursor(0,0);
}

void init_preset(byte mybank,byte mypreset)
{
int i;
int myaddress=0;
char bufnum[4];
int n;
    // Fabrication dun nom bidon au format
    // "PRESET xxx-Bx   "
    strcpy(nompreset,"PRESET ");
    if (mypreset+1<10) {
        strcat(nompreset,"00");
    }
    else {
      if (mypreset+1<100) {
        strcat(nompreset,"0");
      }
    }
    itoa(mypreset+1,bufnum,10);
    strcat(nompreset,bufnum);
    strcat(nompreset,"-B");
    
    itoa(mybank,bufnum,10);
    strcat(nompreset,bufnum);
    strcat(nompreset,"   ");
    // Initialisation de l'adresse du preset
    myaddress=(mybank*BANKSIZE)+(mypreset*PRESETSIZE);
    for (i=0;i<16;i++) {
      eewrite_byte( 0x50, myaddress++, nompreset[i]);
    }
    // Tuning cooarse
    eewrite_byte( 0x50, myaddress++, coarse1);
    eewrite_byte( 0x50, myaddress++, coarse2);
    eewrite_byte( 0x50, myaddress++, coarse3);
    // Tuning fine
    eewrite_byte( 0x50, myaddress++, fine1);
    eewrite_byte( 0x50, myaddress++, fine2);
    eewrite_byte( 0x50, myaddress++, fine3);
    // PWM
    eewrite_byte( 0x50, myaddress++, highByte(pw1));
    eewrite_byte( 0x50, myaddress++, lowByte(pw1));
    eewrite_byte( 0x50, myaddress++, highByte(pw2));
    eewrite_byte( 0x50, myaddress++, lowByte(pw2));
    eewrite_byte( 0x50, myaddress++, highByte(pw3));
    eewrite_byte( 0x50, myaddress++, lowByte(pw3));
    // Controle
    eewrite_byte( 0x50, myaddress++, ctrl1);
    eewrite_byte( 0x50, myaddress++, ctrl2);
    eewrite_byte( 0x50, myaddress++, ctrl3);
    // Attaque, decay
    eewrite_byte( 0x50, myaddress++, ad1);
    eewrite_byte( 0x50, myaddress++, ad2);
    eewrite_byte( 0x50, myaddress++, ad3);
    // Sustain, release
    eewrite_byte( 0x50, myaddress++, sr1);
    eewrite_byte( 0x50, myaddress++, sr2);
    eewrite_byte( 0x50, myaddress++, sr3);
    // Frequence filtre
    eewrite_byte( 0x50, myaddress++, highByte(freq));
    eewrite_byte( 0x50, myaddress++, lowByte(freq));
    // Resonnance filtre
    eewrite_byte( 0x50, myaddress++, res);
    // Mode filtre et volume
    eewrite_byte( 0x50, myaddress++, mode);
    // Future gestion LFO
    eewrite_byte( 0x50, myaddress++, Lfo1Speed);
    eewrite_byte( 0x50, myaddress++, Lfo2Speed);
    eewrite_byte( 0x50, myaddress++, Lfo1Wave);
    eewrite_byte( 0x50, myaddress++, Lfo2Wave);
    eewrite_byte( 0x50, myaddress++, Lfo1Dest);
    eewrite_byte( 0x50, myaddress++, Lfo2Dest);
    eewrite_byte( 0x50, myaddress++, Lfo1Level);
    eewrite_byte( 0x50, myaddress++, Lfo2Level);
}

void loop() 
{
int addr=0; // adresse de debut
int bank;   // Numero de bank
int i,j,k;  // variables de travail
    lcd.setCursor(0,0);
    lcd.print("Init. EEPROM...");
    lcd.setCursor(0,1);
    lcd.print("Bank....:");
    lcd.setCursor(0,2);
    lcd.print("Preset..:");
    for (i=0;i<MAXBANK;i++) {
        lcd.setCursor(10,1);
        lcd.print(i,DEC);
        for (j=0;j<MAXPROG;j++) {
          lcd.setCursor(10,2);
          lcd.print(j+1,DEC);
          lcd.print("   ");
          init_preset(i,j);
        }
    }
    lcd.clear();
    lcd.print("Fin de l'ecriture");
    while(1) {
    }
}